'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var prevContext = null;
var currentContext = null;

var getCurrentContext = function () { return currentContext; };
var setCurrentContext = function (ctx) {
  prevContext = currentContext;
  currentContext = ctx;
}
var revertContext = function () {
  currentContext = prevContext;
}

var createContext = function (label) {
  var ctx = {};
  var disposables = [];
  var state = {};

  var run = function (computation) {
    prevContext = currentContext;
    currentContext = ctx;
    computation();
    currentContext = prevContext;
  }

  var addDisposable = function (disposable) { return disposables.push(disposable); };

  var dispose = function () {
    disposables.forEach(function (fn) { return fn(); });
  }

  var getState = function () { return state; };

  // public API
  ctx.run = run;
  ctx.addDisposable = addDisposable;
  ctx.dispose = dispose;
  ctx.getState = getState;
  return ctx;
}

var withContext = function (fn) { return function () {
  var this$1 = this;
  var params = [], len = arguments.length;
  while ( len-- ) params[ len ] = arguments[ len ];

  var ctx = createContext();
  return ctx.run(function () {
    fn.apply(this$1, params);
  });
}; }

var native = {
  Timeout: global.setTimeout,
  Interval: global.setInterval,
}

var unpatch = function () {
  global.setTimeout = native.Timeout;
  global.setInteval = native.Interval;
};

var patch = function () {
  var methods = ['Timeout', 'Interval'];

  methods.forEach(function (method) { return global[("set" + method)] = function (fn, ms) {
      var params = [], len = arguments.length - 2;
      while ( len-- > 0 ) params[ len ] = arguments[ len + 2 ];

      var currentContext = getCurrentContext();
      if (!currentContext) {
        return native[method].apply(native, [ fn, ms ].concat( params ));
      }

      var computation = function () {
        setCurrentContext(currentContext);
        fn.apply(void 0, params);
        revertContext();
      }
      var id = native[method](computation, ms);
      var dispose = function () { return global[("clear" + method)](id); };
      currentContext.addDisposable(dispose);
      return id;
    }; }
  );

  return unpatch;
}

patch();

exports.getCurrentContext = getCurrentContext;
exports.setCurrentContext = setCurrentContext;
exports.revertContext = revertContext;
exports.createContext = createContext;
exports.withContext = withContext;